@@ -61,12 +61,9 @@ extern pid_t trace_pid;
 extern void sb_lock(void);
 extern void sb_unlock(void);
 
+bool trace_possible(const char *filename, char *const argv[], const void *data);
 void trace_main(const char *filename, char *const argv[]);
 
-__attribute__((__format__(__printf__, 1, 2))) void sb_eqawarn(const char *format, ...);
-void sb_dump_backtrace(void);
-__attribute__((noreturn)) void sb_abort(void);
-
 /* glibc modified realpath() function */
 char *erealpath(const char *, char *);
 char *egetcwd(char *, size_t);
