@@ -11,15 +11,16 @@
 
 static long _do_ptrace(enum __ptrace_request request, const char *srequest, void *addr, void *data);
 #define do_ptrace(request, addr, data) _do_ptrace(request, #request, addr, data)
+#define _trace_possible(data) true
 
 #ifdef DEBUG
 # define SBDEBUG 1
 #else
 # define SBDEBUG 0
 #endif
-#define __SB_DEBUG(fmt, args...) do { if (SBDEBUG) sb_printf(fmt, ## args); } while (0)
-#define _SB_DEBUG(fmt, args...)  do { if (SBDEBUG) SB_EWARN("TRACE ", "(pid=%i):%s: " fmt, getpid(), __func__, ## args); } while (0)
-#define SB_DEBUG(fmt, args...)   _SB_DEBUG(fmt "\n", ## args)
+#define __sb_debug(fmt, args...) do { if (SBDEBUG) sb_printf(fmt, ## args); } while (0)
+#define _sb_debug(fmt, args...)  do { if (SBDEBUG) sb_ewarn("TRACE (pid=%i):%s: " fmt, getpid(), __func__, ## args); } while (0)
+#define sb_debug(fmt, args...)   _sb_debug(fmt "\n", ## args)
 
 #include "trace/os.c"
 
@@ -27,6 +28,10 @@ pid_t trace_pid;
 
 #ifndef SB_NO_TRACE
 
+#ifndef HAVE_TRACE_REGS
+# error "unable to find struct for tracing regs"
+#endif
+
 #ifdef HAVE_OPEN64
 # define sb_openat_pre_check sb_openat64_pre_check
 #endif
@@ -60,18 +65,12 @@ static long _do_ptrace(enum __ptrace_request request, const char *srequest, void
 			    request == PTRACE_PEEKUSER)
 				return ret;
 
-		SB_EERROR("ISE:_do_ptrace ", "ptrace(%s, ..., %p, %p): %s\n",
+		sb_ebort("ISE:_do_ptrace: ptrace(%s, ..., %p, %p): %s\n",
 			srequest, addr, data, strerror(errno));
-		sb_abort();
 	}
 	return ret;
 }
 
-static long do_peekuser(long offset)
-{
-	return do_ptrace(PTRACE_PEEKUSER, (void *)offset, NULL);
-}
-
 static long do_peekdata(long offset)
 {
 	return do_ptrace(PTRACE_PEEKDATA, (void *)offset, NULL);
@@ -149,7 +148,7 @@ static const char *strsig(int sig)
 
 static void trace_child_signal(int signo, siginfo_t *info, void *context)
 {
-	SB_DEBUG("got sig %s(%i): code:%s(%i) status:%s(%i)",
+	sb_debug("got sig %s(%i): code:%s(%i) status:%s(%i)",
 		strsig(signo), signo,
 		strcld_chld(info->si_code), info->si_code,
 		strsig(info->si_status), info->si_status);
@@ -160,7 +159,7 @@ static void trace_child_signal(int signo, siginfo_t *info, void *context)
 			trace_exit(128 + info->si_status);
 
 		case CLD_EXITED:
-			__SB_DEBUG(" = %i\n", info->si_status);
+			__sb_debug(" = %i\n", info->si_status);
 			trace_exit(info->si_status);
 
 		case CLD_TRAPPED:
@@ -181,7 +180,7 @@ static void trace_child_signal(int signo, siginfo_t *info, void *context)
 			return;
 	}
 
-	SB_EERROR("ISE:trace_child_signal ", "child (%i) signal %s(%i), code %s(%i), status %s(%i)\n",
+	sb_eerror("ISE:trace_child_signal: child (%i) signal %s(%i), code %s(%i), status %s(%i)\n",
 		trace_pid,
 		strsig(signo), signo,
 		strcld_chld(info->si_code), info->si_code,
@@ -197,10 +196,6 @@ static const struct syscall_entry *lookup_syscall_in_tbl(const struct syscall_en
 			++tbl;
 	return NULL;
 }
-static const struct syscall_entry *lookup_syscall(int nr)
-{
-	return lookup_syscall_in_tbl(trace_check_personality(), nr);
-}
 
 struct syscall_state {
 	void *regs;
@@ -212,7 +207,7 @@ struct syscall_state {
 static bool _trace_check_syscall_C(struct syscall_state *state, int ibase)
 {
 	char *path = do_peekstr(trace_arg(state->regs, ibase));
-	__SB_DEBUG("(\"%s\")", path);
+	__sb_debug("(\"%s\")", path);
 	bool pre_ret, ret;
 	if (state->pre_check)
 		pre_ret = state->pre_check(state->func, path, AT_FDCWD);
@@ -234,7 +229,7 @@ static bool __trace_check_syscall_DCF(struct syscall_state *state, int ibase, in
 {
 	int dirfd = trace_arg(state->regs, ibase);
 	char *path = do_peekstr(trace_arg(state->regs, ibase + 1));
-	__SB_DEBUG("(%i, \"%s\", %x)", dirfd, path, flags);
+	__sb_debug("(%i, \"%s\", %x)", dirfd, path, flags);
 	bool pre_ret, ret;
 	if (state->pre_check)
 		pre_ret = state->pre_check(state->func, path, dirfd);
@@ -331,7 +326,7 @@ static bool trace_check_syscall(const struct syscall_entry *se, void *regs)
 	else if (nr == SB_NR_ACCESS) {
 		char *path = do_peekstr(trace_arg(regs, 1));
 		int flags = trace_arg(regs, 2);
-		__SB_DEBUG("(\"%s\", %x)", path, flags);
+		__sb_debug("(\"%s\", %x)", path, flags);
 		ret = _SB_SAFE_ACCESS(nr, name, path, flags);
 		free(path);
 		return ret;
@@ -340,7 +335,7 @@ static bool trace_check_syscall(const struct syscall_entry *se, void *regs)
 		int dirfd = trace_arg(regs, 1);
 		char *path = do_peekstr(trace_arg(regs, 2));
 		int flags = trace_arg(regs, 3);
-		__SB_DEBUG("(%i, \"%s\", %x)", dirfd, path, flags);
+		__sb_debug("(%i, \"%s\", %x)", dirfd, path, flags);
 		ret = _SB_SAFE_ACCESS_AT(nr, name, dirfd, path, flags);
 		free(path);
 		return ret;
@@ -348,7 +343,7 @@ static bool trace_check_syscall(const struct syscall_entry *se, void *regs)
 	} else if (nr == SB_NR_OPEN) {
 		char *path = do_peekstr(trace_arg(regs, 1));
 		int flags = trace_arg(regs, 2);
-		__SB_DEBUG("(\"%s\", %x)", path, flags);
+		__sb_debug("(\"%s\", %x)", path, flags);
 		if (sb_openat_pre_check(name, path, AT_FDCWD, flags))
 			ret = _SB_SAFE_OPEN_INT(nr, name, path, flags);
 		else
@@ -360,7 +355,7 @@ static bool trace_check_syscall(const struct syscall_entry *se, void *regs)
 		int dirfd = trace_arg(regs, 1);
 		char *path = do_peekstr(trace_arg(regs, 2));
 		int flags = trace_arg(regs, 3);
-		__SB_DEBUG("(%i, \"%s\", %x)", dirfd, path, flags);
+		__sb_debug("(%i, \"%s\", %x)", dirfd, path, flags);
 		if (sb_openat_pre_check(name, path, dirfd, flags))
 			ret = _SB_SAFE_OPEN_INT_AT(nr, name, dirfd, path, flags);
 		else
@@ -370,48 +365,37 @@ static bool trace_check_syscall(const struct syscall_entry *se, void *regs)
 	}
 
  done:
-	__SB_DEBUG("(...)");
+	__sb_debug("(...)");
 	return ret;
 }
 
-/* Some arches (like hppa) don't implement PTRACE_GETREGS, while others (like
- * sparc) swap the meaning of "addr" and "data.  What a bunch of asshats.
- */
-#ifndef trace_get_regs
-# define trace_get_regs(regs) do_ptrace(PTRACE_GETREGS, NULL, regs)
-#endif
-/* Some arches (like sparc) don't implement PTRACE_PEEK* ...
- * more asshats !
- */
-#ifndef trace_sysnum_regs
-# define trace_sysnum_regs(regs) trace_sysnum()
-#endif
-
 static void trace_loop(void)
 {
 	trace_regs regs;
-	bool before_syscall;
+	bool before_syscall, fake_syscall_ret;
 	long ret;
 	int nr, exec_state;
-	const struct syscall_entry *se, *tbl_at_fork;
+	const struct syscall_entry *se, *tbl_at_fork, *tbl_after_fork;
 
 	exec_state = 0;
 	before_syscall = true;
-	tbl_at_fork = NULL;
+	fake_syscall_ret = false;
+	tbl_at_fork = tbl_after_fork = NULL;
 	do {
 		ret = do_ptrace(PTRACE_SYSCALL, NULL, NULL);
 		waitpid(trace_pid, NULL, 0);
-		nr = trace_sysnum_regs(&regs);
+		ret = trace_get_regs(&regs);
+		nr = trace_get_sysnum(&regs);
 
 		if (!exec_state) {
 			if (!tbl_at_fork)
-				tbl_at_fork = trace_check_personality();
+				tbl_at_fork = trace_check_personality(&regs);
 			se = lookup_syscall_in_tbl(tbl_at_fork, nr);
 			if (!before_syscall || !se || se->sys != SB_NR_EXECVE) {
 				if (before_syscall)
-					_SB_DEBUG(">%s:%i", se ? se->name : "IDK", nr);
+					_sb_debug(">%s:%i", se ? se->name : "IDK", nr);
 				else
-					__SB_DEBUG("(...pre-exec...) = ...\n");
+					__sb_debug("(...pre-exec...) = ...\n");
 				goto loop_again;
 			}
 			++exec_state;
@@ -421,24 +405,31 @@ static void trace_loop(void)
 			goto loop_again;
 		}
 
-		se = lookup_syscall(nr);
+		if (!tbl_after_fork)
+			tbl_after_fork = trace_check_personality(&regs);
+		se = lookup_syscall_in_tbl(tbl_after_fork, nr);
 		ret = trace_get_regs(&regs);
 		if (before_syscall) {
-			_SB_DEBUG("%s:%i", se ? se->name : "IDK", nr);
+			_sb_debug("%s:%i", se ? se->name : "IDK", nr);
 			if (!trace_check_syscall(se, &regs)) {
-				if (is_env_on(ENV_SANDBOX_DEBUG))
-					SB_EINFO("trace_loop", " destroying after %s\n",
-						se->name);
-				do_ptrace(PTRACE_KILL, NULL, NULL);
-				exit(1);
+				sb_debug_dyn("trace_loop: forcing EPERM after %s\n", se->name);
+				trace_set_sysnum(&regs, -1);
+				fake_syscall_ret = true;
 			}
 		} else {
 			int err;
-			ret = trace_result(&regs, &err);
 
-			__SB_DEBUG(" = %li", ret);
+			if (unlikely(fake_syscall_ret)) {
+				ret = -1;
+				err = EPERM;
+				trace_set_ret(&regs, err);
+				fake_syscall_ret = false;
+			} else
+				ret = trace_result(&regs, &err);
+
+			__sb_debug(" = %li", ret);
 			if (err) {
-				__SB_DEBUG(" (errno: %i: %s)", err, strerror(err));
+				__sb_debug(" (errno: %i: %s)", err, strerror(err));
 
 				/* If the exec() failed for whatever reason, kill the
 				 * child and have the parent resume like normal
@@ -449,7 +440,7 @@ static void trace_loop(void)
 					return;
 				}
 			}
-			__SB_DEBUG("\n");
+			__sb_debug("\n");
 
 			exec_state = 2;
 		}
@@ -467,21 +458,16 @@ void trace_main(const char *filename, char *const argv[])
 	sa.sa_sigaction = trace_child_signal;
 	sigaction(SIGCHLD, &sa, &old_sa);
 
-	if (is_env_on(ENV_SANDBOX_DEBUG))
-		SB_EINFO("trace_main", " tracing: %s\n", filename);
+	sb_debug_dyn("trace_main: tracing: %s\n", filename);
 
-	if (trace_pid) {
-		SB_EERROR("ISE:trace_main ", "trace code assumes multiple threads are not forking\n");
-		sb_abort();
-	}
+	if (trace_pid)
+		sb_ebort("ISE: trace code assumes multiple threads are not forking\n");
 
 	trace_pid = fork();
-	if (trace_pid == -1) {
-		SB_EERROR("ISE:trace_main ", "vfork() failed: %s\n",
-			strerror(errno));
-		sb_abort();
+	if (unlikely(trace_pid == -1)) {
+		sb_ebort("ISE: vfork() failed: %s\n", strerror(errno));
 	} else if (trace_pid) {
-		SB_DEBUG("parent waiting for child (pid=%i) to signal", trace_pid);
+		sb_debug("parent waiting for child (pid=%i) to signal", trace_pid);
 		waitpid(trace_pid, NULL, 0);
 #if defined(PTRACE_SETOPTIONS) && defined(PTRACE_O_TRACESYSGOOD)
 		/* Not all kernel versions support this, so ignore return */
@@ -491,7 +477,7 @@ void trace_main(const char *filename, char *const argv[])
 		return;
 	}
 
-	SB_DEBUG("child setting up ...");
+	sb_debug("child setting up ...");
 	sigaction(SIGCHLD, &old_sa, NULL);
 	do_ptrace(PTRACE_TRACEME, NULL, NULL);
 	kill(getpid(), SIGSTOP);
@@ -500,6 +486,16 @@ void trace_main(const char *filename, char *const argv[])
 
 #else
 
+#undef _trace_possible
+#define _trace_possible(data) false
+
+void trace_main(const char *filename, char *const argv[])
+{
+	/* trace_possible() triggers a warning for us */
+}
+
+#endif
+
 static char *flatten_args(char *const argv[])
 {
 	char *ret;
@@ -527,11 +523,13 @@ static char *flatten_args(char *const argv[])
 	return ret;
 }
 
-void trace_main(const char *filename, char *const argv[])
+bool trace_possible(const char *filename, char *const argv[], const void *data)
 {
-	char *args = flatten_args(argv);
-	sb_eqawarn("Static ELF: %s: %s\n", filename, args);
-	free(args);
-}
+	if (_trace_possible(data))
+		return true;
 
-#endif
+	char *args = flatten_args(argv);
+	sb_eqawarn("Unable to trace static ELF: %s: %s\n", filename, args);
+	free(args);
+	return false;
+}
